; -*-Emacs-Lisp-*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File:         winterpmode.el
; RCS:          $Header: winterpmode.el,v 2.3 93/04/14 04:33:01 mayer Exp $
; Description:  Stuff for running WINTERP in a subshell
; Author:       Niels Mayer, HPLabs
; Created:      Thu Jun  8 18:16:00 1989
; Modified:     Sun Sep 20 19:37:59 1992 (Niels Mayer) mayer@hplmmws
; Language:     Emacs-Lisp
; Package:      N/A
; Status:       Experimental (Do Not Distribute)
;
; (c) Copyright 1988, Hewlett-Packard Company, all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(require 'shell)			; I'm assuming this is the standard
					; 18.54 version of shell.el.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Code to run 'winterp' process as a gnuemacs subshell. This is essentially
;;; the same as the code to run a common lisp process as a gnuemacs subshell.
;;; However, with this code, you may run other lisps alongside winterp.
;;; (code stolen & mutated from emacs/lisp/shell.el)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar inferior-winterp-mode-map nil)
(if inferior-winterp-mode-map
    nil
  (setq inferior-winterp-mode-map (copy-alist shell-mode-map))
  (lisp-mode-commands inferior-winterp-mode-map)
  (define-key inferior-winterp-mode-map "\e\C-x"	'winterp-send-defun)
  (define-key inferior-winterp-mode-map "\C-c\C-l"	'run-winterp)
  (define-key inferior-winterp-mode-map "\C-c\C-d"	'winterp-send-exit-brklevel)
  )

(defvar inferior-winterp-program "winterp"
  "*Program name for invoking an inferior Lisp with `run-winterp'.")

(defvar inferior-winterp-load-command
  "(load \"%s\" :verbose nil :print t)\n"
  "*Format-string for building a Winterp expression to load a file.
This format string should use %s to substitute a file name
and should result in a Winterp expression that will command the inferior Winterp
to load that file.  The default works acceptably on most Lisps.
The string \"(progn (load \\\"%s\\\" :verbose nil :print t) (values))\\\n\"
produces cosmetically superior output for this application,
but it works only in Common Lisp.")

(defvar inferior-winterp-prompt "^[X0-9]> $"
  "*Regexp to recognize prompts from the inferior Lisp.
\"^.*>:? *$\" is right for Franz Lisp and kcl.")

(defun inferior-winterp-mode ()
  "Major mode for interacting with an inferior Winterp process.
Runs a Winterp interpreter as a subprocess of Emacs, with Winterp I/O
through an Emacs buffer.  Variable inferior-winterp-program controls
which Winterp interpreter is run.  Variables inferior-winterp-prompt
and inferior-winterp-load-command can customize this mode for different
Lisp interpreters.

Commands:
DELETE converts tabs to spaces as it moves back.
TAB indents for Lisp; with argument, shifts rest
 of expression rigidly with the current line.
Meta-Control-Q does TAB on each line starting within following expression.
Paragraphs are separated only by blank lines.  Semicolons start comments.

Return at end of buffer sends line as input.
Return not at end copies rest of line to end and sends it.

The following commands imitate the usual Unix interrupt and
editing control characters... (note the addition of 
\\[winterp-send-defun], \\[winterp-send-exit-brklevel], and \\[run-winterp].)
\\{shell-mode-map}

Entry to this mode calls the value of winterp-mode-hook with no arguments,
if that value is non-nil.  Likewise with the value of shell-mode-hook.
winterp-mode-hook is called after shell-mode-hook.

You can send text to the inferior Winterp from other buffers
using the commands process-send-region, process-send-string,
winterp-send-defun (\\[winterp-send-defun]),
winterp-send-buffer (ESC C-z),
and winterp-send-exit-brklevel (\\[winterp-send-exit-brklevel])."
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'inferior-winterp-mode)
  (setq mode-name "Inferior Winterp")
  (setq mode-line-process '(": %s"))
  (lisp-mode-variables t)
  (use-local-map inferior-winterp-mode-map)
  (make-local-variable 'last-input-start)
  (setq last-input-start (make-marker))
  (make-local-variable 'last-input-end)
  (setq last-input-end (make-marker))
  (run-hooks 'shell-mode-hook 'winterp-mode-hook))

(defun run-winterp ()
  "Run an inferior Winterp process, input and output via buffer *winterp*."
  (interactive)
  (switch-to-buffer (make-shell "winterp" inferior-winterp-program))
  (inferior-winterp-mode))

(defun winterp-send-defun (no-display-flag)
  "Send the current defun to the Winterp process made by M-x run-winterp.
With argument, suppress redisplay and scrolling of the *winterp* buffer.
Variable `inferior-winterp-load-command' controls formatting of
the `load' form that is set to the Winterp process."
  (interactive "P")
  (or (get-process "winterp")
      (error "No current winterp process"))
  (save-excursion
   (end-of-defun)
   (let ((end (point))
	 (filename (format "/tmp/wl%d.lsp" (process-id (get-process "winterp")))))
     (beginning-of-defun)
     (write-region (point) end filename nil 'nomessage)
     (process-send-string "winterp" (format inferior-winterp-load-command filename)))
   (if (not no-display-flag)
       (let* ((process (get-process "winterp"))
	      (buffer (process-buffer process))
	      (w (or (get-buffer-window buffer) (display-buffer buffer)))
	      (height (window-height w))
	      (end))
	 (save-excursion
	   (set-buffer buffer)
	   (setq end (point-max))
	   (while (progn
		    (accept-process-output process)
		    (goto-char (point-max))
		    (beginning-of-line)
		    (or (= (point-max) end)
			(not (looking-at inferior-winterp-prompt)))))
	   (setq end (point-max))
	   (vertical-motion (- 4 height))
	   (set-window-start w (point)))
	 (set-window-point w end)))))

(defun winterp-send-defun-and-go ()
  "Send the current defun to the inferior Winterp, and switch to *winterp* buffer."
  (interactive)
  (winterp-send-defun t)
  (switch-to-buffer "*winterp*"))

(defun winterp-send-buffer (no-display-flag)
  "A version of winterp-send-defun that sends the entire buffer off for 
evaluation (by the Winterp process made by M-x run-winterp).
With argument, suppress redisplay and scrolling of the *winterp* buffer.
Variable `inferior-winterp-load-command' controls formatting of
the `load' form that is set to the Winterp process."
  (interactive "P")
  (if (not (get-process "winterp"))
      (error "No current winterp process")
    )
  (let ((loadfile (format "/tmp/wl%d.lsp"
			  (process-id (get-process "winterp")))))
    (save-excursion (write-region (point-min) (point-max)
				  loadfile nil 'nomessage))
    (process-send-string "winterp"
			 (format inferior-winterp-load-command loadfile))
    )

  (if (not no-display-flag)
      (let* ((process (get-process "winterp"))
	     (buffer (process-buffer process))
	     (w (or (get-buffer-window buffer) (display-buffer buffer)))
	     (height (window-height w))
	     (end))
	(save-excursion
	  (set-buffer buffer)
	  (setq end (point-max))
	  (while (progn
		   (accept-process-output process)
		   (goto-char (point-max))
		   (beginning-of-line)
		   (or (= (point-max) end)
		       (not (looking-at inferior-winterp-prompt)))))
	  (setq end (point-max))
	  (vertical-motion (- 4 height))
	  (set-window-start w (point)))
	(set-window-point w end)))
  )

(defun winterp-send-exit-brklevel (no-display-flag)
  "Calling this function will send WINTERP a command to exit the current
breaklevel. If not at a breaklevel, WINTERP will ignore the input..."
  (interactive "P")
  (if (not (get-process "winterp"))
      (error "No current winterp process")
    )
  (process-send-string "winterp" "")

  (if (not no-display-flag)
      (let* ((process (get-process "winterp"))
	     (buffer (process-buffer process))
	     (w (or (get-buffer-window buffer) (display-buffer buffer)))
	     (height (window-height w))
	     (end))
	(save-excursion
	  (set-buffer buffer)
	  (setq end (point-max))
	  (while (progn
		   (accept-process-output process)
		   (goto-char (point-max))
		   (beginning-of-line)
		   (or (= (point-max) end)
		       (not (looking-at inferior-winterp-prompt)))))
	  (setq end (point-max))
	  (vertical-motion (- 4 height))
	  (set-window-start w (point)))
	(set-window-point w end)))
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; This code sets up winterp-mode when visiting a file with extension .lsp.
;;; This is exactly the same as gnuemacs' normal lisp mode; I just wanted
;;; to avoid conflict with other lisps that you might be using under gnuemacs,
;;; such as common lisp.
;;; The following was stolen&mutated from emacs/lisp/lisp-mode.el
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar winterp-mode-map ())
(if winterp-mode-map
    ()
  (setq winterp-mode-map (make-sparse-keymap))
  (define-key winterp-mode-map "\e\C-x"		'winterp-send-defun)
  (define-key winterp-mode-map "\C-c\C-l"	'run-winterp)
  (define-key winterp-mode-map "\C-c\C-d"	'winterp-send-exit-brklevel)
  (define-key winterp-mode-map "\e\C-z"		'winterp-send-buffer)
  (lisp-mode-commands winterp-mode-map)
  )

(defun winterp-mode ()
  "Major mode for editing Winterp code.
Commands:
Delete converts tabs to spaces as it moves back.
Blank lines separate paragraphs.  Semicolons start comments.
\\{winterp-mode-map}
Note that `run-winterp' may be used either to start an inferior Winterp job
or to switch back to an existing one.

Entry to this mode calls the value of winterp-mode-hook
if that value is non-nil."
  (interactive)
  (kill-all-local-variables)
  (use-local-map winterp-mode-map)
  (setq major-mode 'winterp-mode)
  (setq mode-name "Winterp-Lisp")
  (lisp-mode-variables t)
  (set-syntax-table lisp-mode-syntax-table)
  (run-hooks 'winterp-mode-hook))

; ;;
; ;; All files with extension .lsp are assumed to be WINTERP files, to be edited in
; ;; winterp-mode.
; ;; Note -- this overrides a binding in loaddefs.el which will put files with
; ;; extension .lsp in lisp-mode
; ;;
(setq auto-mode-alist
      (append '(("\\.lsp$" . winterp-mode)
		)
	      auto-mode-alist))
